/**
 * \file: myprime_serial.cpp (Without MPI C++ API)
 *
 * serial, but MPI implementation of Eratosthenes's sieve
 * It could be used as a skeleton for data-paralell implementation
 *
 */

#include <iostream>
#include <mpi.h>
#include <vector>
#include "myutils.hpp"


// Indexable array for storing bits.
// Default value is 0
// Index range: [low..high]
// Out-of-rage indexing returns 0
class BitArray {
    size_t low;		// low index value
    size_t high;	// high index value
    std::vector<bool> s;// bitvector
    std::vector<bool>::reference dummy;	// used as lvalue at out of range
public:
    BitArray(size_t low, size_t high) :low(low), high(high), s(high-low+1) {}
    bool operator[](size_t i) const {
      if (i >= low && i <= high) return s[i-low];
      else return false;
    }
    std::vector<bool>::reference operator[](size_t i) {
      if (i >= low && i <= high) return s[i-low];
      else return dummy;
    }
};

#define N 100
int main(int argc, char ** argv) {
    debMsg cout(std::cout);	// this is the own cout for debugging
    size_t n = N;
    if (argc > 1) n = atoi(argv[1]);

    MPI_Init(&argc, &argv);
    int rank, size;
    MPI_Comm_rank(MPI_COMM_WORLD, &rank);
    MPI_Comm_size(MPI_COMM_WORLD, &size);
    // set the debug id
    {
      std::stringstream id;
      id << "PROC" << rank << "> ";
      cout.setid(id.str());
    }
    Stopwatch watch;

    BitArray V(2, n);
    size_t m = 2;

    while (m*m < n) {
      if (V[m] == false) {
        size_t i = m*m;
        while (i <= n) {
          V[i] = true;
          i = i+m;
        }
      }
      m = m+1;
    }
    // The results should be collected from parallel parts
    //  ...

    // Writes elapsed time to separate stream (to cerr)
    std::cerr << "Elapsed time:" << watch.elapsed() << "sec" << endl;

    // Prints the resuls
    cout << "Primes in [1.." << n << "]:\n";
    if (argc <= 2) {	// argc >2 supress the output
      for (size_t i = 2; i <= n; i++)
        if (V[i] == false)
          cout << " " << i;
    }
    cout << endl;
    MPI_Finalize();
    return 0;
}
