/**
 * \file: myutils.hpp
 *
 * usleep function
 * debMsg class
 *
 */
#ifndef MYUTILS_INLUDED
#define MYUTULS_INLUDED

#include <iostream>
#include <sstream>
#include <string>
#include <unistd.h>
#include <chrono>
#include <thread>

/// microsecond sleep
/// @param t - time to sleep in useconds
void uSleep(int usec) {
    std::this_thread::sleep_for(std::chrono::microseconds(usec));
}

/// debMsg class for debugging.
/// Collects the output and inserts to the stream when the flush() is called.
/// In this way the outputs of differnt parallel tasks will not mixed (hopefully).
class debMsg : public std::stringstream {
    std::ostream& os;
    std::string id;
public:
    debMsg(std::ostream& s = std::cerr, const std::string& id = "")
	    						:os(s), id(id) {
	*this << id;
	std::setvbuf(stdout, NULL, _IOFBF, BUFSIZ);
    }
    void flush() {
	if (this->str() == id) return;
	(os << this->str()).flush();
	this->str("");	// clear the string
	*this << id;
    }
    void setid(const std::string& id = "") {
	this->id = id;
	flush();
    }
    ~debMsg() { flush(); }
};

/// My endl
class endl_t {} endl;

/// My endline inserted to the stream
std::ostream& operator<<(std::ostream& os, endl_t) {
    os.put('\n');	// newline
    try {
        dynamic_cast<debMsg&>(os).flush(); // convertable -> calls flush()
    } catch (std::bad_cast&) {
        os.flush();	// can't cast
    }
    return os;
}

class Stopwatch {
  double sec0;
public:
  Stopwatch() : sec0(MPI_Wtime()) {}
  double elapsed() {
    double sec1 = MPI_Wtime();
    double sec = sec1 - sec0;
    sec0 = sec1;
    return sec;
  }
};

#endif
